@@ -1,79 +1,28 @@
 package CGI::Auth::FOAF_SSL;
 
-use 5.008;
+use 5.010;
 use common::sense;
 
-=head1 NAME
+use CGI::Auth::FOAF_SSL::Agent 0;
+use CGI 0;
+use CGI::Session 0;
+use Crypt::X509 0.50;
+use DateTime 0;
+use File::Spec 0;
+use LWP::UserAgent 0;
+use Math::BigInt 0 try => 'GMP';
+use MIME::Base64 0 qw[];
+use Object::ID 0;
+use RDF::TrineX::Functions -shortcuts;
+use Scalar::Util 0 qw[blessed];
 
-CGI::Auth::FOAF_SSL - authentication using WebID (FOAF+SSL)
-
-=head1 SYNOPSIS
-
-  use CGI qw(:all);
-  use CGI::Auth::FOAF_SSL;
-  
-  my $auth = CGI::Auth::FOAF_SSL->new_from_cgi;
-  
-  print header(-type=>'text/html', -cookie=>$auth->cookie);
-  
-  if (defined $auth && $auth->is_secure)
-  {
-    if (defined $auth->subject)
-    {
-      printf("<p>Hello <a href='%s'>%s</a>!</p>\n",
-             escapeHTML($auth->subject->homepage),
-             escapeHTML($auth->subject->name));
-    }
-    else
-    {
-      print "<p>Hello!</p>\n";
-    }
-  }
-  else
-  {
-    print "<p>Greetings stranger!</p>\n";
-  }
-
-=head1 VERSION
-
-1.001 (developer preview)
-
-=head1 DESCRIPTION
-
-FOAF+SSL (a.k.a. WebID) is a simple authentication scheme described
-at L<http://esw.w3.org/topic/foaf+ssl>. This module implements the server
-end of FOAF+SSL in Perl.
-
-It is suitable for handling authentication using FOAF+SSL over HTTPS.
-Your web server needs to be using HTTPS, configured to request client
-certificates, and make the certificate PEM available to your script. If you
-are using Apache, this means that you want to set the following
-directives in your SSL virtual host setup:
-
- SSLEngine on
- SSLVerifyClient optional_no_ca
- SSLVerifyDepth  1
- SSLOptions +StdEnvVars +ExportCertData
-
-=cut
-
-use CGI::Auth::FOAF_SSL::Agent;
-use CGI;
-use CGI::Session;
-use Crypt::X509 '0.50';
-use DateTime;
-use File::Spec;
-use LWP::UserAgent;
-use Math::BigInt try => 'GMP';
-use MIME::Base64 qw[];
-use RDF::TrineShortcuts '0.100';
-use Scalar::Util qw[blessed refaddr];
+# use Data::Printer 0;
 
 use constant {
 	VALIDATION_PEM     => 1,
 	VALIDATION_DATES   => 2,
 	VALIDATION_WEBID   => 3,
-	};
+};
 	
 our $VERSION;
 our $ua_string;
@@ -83,10 +32,11 @@ my ($AGENT, $MODEL, $SESSION); # inside-out objects
 
 BEGIN
 {
-	$VERSION = '1.001';
-	$ua_string = sprintf('%s/%s ', __PACKAGE__, $VERSION);	
-
+	$VERSION = '1.003';
+	$ua_string = sprintf('%s/%s ', __PACKAGE__, $VERSION);
+	
 	$WWW_Finger = 0;
+	if (0) # DISABLED
 	{
 		local $@ = undef;
 		eval
@@ -103,31 +53,32 @@ BEGIN
 	$SESSION = {};
 }
 
-=head2 Configuration
+sub rdf_query
+{
+	my ($sparql, $model) = @_;
+	my $class   = (blessed($model) and $model->isa('RDF::Trine::Model'))
+		? 'RDF::Query'
+		: 'RDF::Query::Client';
+	my $query   = $class->new($sparql);
+	my $results = $query->execute($model);
+	
+	unless ($results)
+	{
+		warn "$sparql";
+		warn "$model";
+		warn $query->error;
+		return;
+	}
 
-=over 4
-
-=item * C<< $CGI::Auth::FOAF_SSL::ua_string = 'MyTool/1.0' >>
-
-Set the User-Agent string for any HTTP requests.
-
-=back
-
-=head2 Constructors
-
-=over 4
-
-=item * C<< new($pem_encoded) >>
-
-Performs FOAF+SSL authentication on a PEM-encoded key. If authentication is
-completely unsuccessful, returns undef. Otherwise, returns a CGI::Auth::FOAF_SSL
-object. Use C<is_secure> to check if authentication was I<completely> successful.
-
-You probably want to use C<new_from_cgi> instead.
-
-(DER encoded certificates should work too.)
-
-=cut
+	if ($results->is_boolean)
+		{ return $results->get_boolean }
+	if ($results->is_bindings)
+		{ return $results }
+	if ($results->is_graph)
+		{ my $m = rdf_parse(); $m->add_hashref($results->as_hashref); return $m }
+	
+	return;
+}
 
 sub new
 {
@@ -138,9 +89,17 @@ sub new
 	return unless $self->validation(VALIDATION_PEM);
 	
 	my $now = DateTime->now;
-	return if defined $self->cert_not_before && $now < $self->cert_not_before;
-	return if defined $self->cert_not_after  && $now > $self->cert_not_after;
-
+	if (defined $self->cert_not_before && $now < $self->cert_not_before)
+	{
+		warn "Certificate isn't valid yet! Try again on " . $self->cert_not_before->iso8601;
+		return;
+	}
+	if (defined $self->cert_not_after  && $now > $self->cert_not_after)
+	{
+		warn "Certificate has expired on " . $self->cert_not_after->iso8601;
+		return;
+	}
+	
 	$self->validation(VALIDATION_DATES);
 	
 	my $verified;
@@ -166,16 +125,6 @@ sub new
 	return $self;
 }
 
-=item * C<< new_from_cgi($cgi_object) >>
-
-Performs FOAF+SSL authentication on a CGI object. This is a wrapper around
-C<new> which extracts the PEM-encoded client certificate from the CGI
-request. It has the same return values as C<new>.
-
-If $cgi_object is omitted, uses C<< CGI->new >> instead.
-
-=cut
-
 sub new_from_cgi
 {
 	my ($class, $cgi, @params) = @_;
@@ -228,17 +177,6 @@ sub new_unauthenticated
 	return $self;
 }
 
-=back
-
-=head2 Public Methods
-
-=over 4
-
-=item * C<< is_secure >>
-
-Returns true iff the FOAF+SSL authentication process was completely successful.
-
-=cut
 
 sub is_secure
 {
@@ -246,42 +184,23 @@ sub is_secure
 	return ($self->validation == VALIDATION_WEBID) ? 1 : 0;
 }
 
-=item * C<< subject >>
-
-Returns a L<CGI::Auth::FOAF_SSL::Agent> object which represents the subject
-of the certificate. 
-
-This method has aliases C<agent> and C<certified_thing> for back-compat
-reasons.
-
-=cut
-
 sub subject
 {
 	my ($self) = @_;
 
-	$AGENT->{ refaddr($self) } ||= CGI::Auth::FOAF_SSL::Agent->new(
+	$AGENT->{ $self->object_id } ||= CGI::Auth::FOAF_SSL::Agent->new(
 		$self->subject_uri,
 		$self->subject_model,
 		$self->subject_endpoint,
 		);
 
-	return $AGENT->{ refaddr($self) };
+	return $AGENT->{ $self->object_id };
 }
 
 *certified_thing = \&subject;
 *agent           = \&subject;
 *account         = sub { return; };
 
-=item * C<< cookie >>
-
-HTTP cookie related to the authentication process. Sending this to the
-client isn't strictly necessary, but it allows for a session to be
-established, greatly speeding up subsequent accesses. See also the
-COOKIES section of this documentation.
-
-=cut
-
 sub cookie
 {
 	my ($self) = @_;
@@ -316,7 +235,7 @@ sub authenticate_by_sparql
 {
 	my ($self, $uri, $model, $fp) = @_;
 	
-	my $query_string = sprintf(<<'SPARQL', $uri);
+	my $query_string = sprintf(<<'SPARQL', (($uri)x4));
 PREFIX cert: <http://www.w3.org/ns/auth/cert#>
 PREFIX rsa: <http://www.w3.org/ns/auth/rsa#>
 SELECT
@@ -326,11 +245,33 @@ SELECT
 	?hexModulus
 WHERE
 {
-	?key
-		cert:identity <%s> ;
-		rsa:modulus ?modulus ;
-		rsa:public_exponent ?exponent .
-
+	{
+		?key
+			cert:identity <%s> ;
+			rsa:modulus ?modulus ;
+			rsa:public_exponent ?exponent .
+	}
+	UNION
+	{
+		<%s> cert:key ?key .
+		?key
+			rsa:modulus ?modulus ;
+			rsa:public_exponent ?exponent .
+	}
+	UNION
+	{
+		?key
+			cert:identity <%s> ;
+			cert:modulus ?modulus ;
+			cert:exponent ?exponent .
+	}
+	UNION
+	{
+		<%s> cert:key ?key .
+		?key
+			cert:modulus ?modulus ;
+			cert:exponent ?exponent .
+	}
 	OPTIONAL { ?modulus cert:hex ?hexModulus . }
 	OPTIONAL { ?exponent cert:decimal ?decExponent . }
 }
@@ -340,6 +281,9 @@ SPARQL
 	
 	RESULT: while (my $result = $results->next)
 	{
+		# trim any whitespace around modulus (HACK for MyProfile WebIDs)
+		$result->{modulus}->[0] =~ s/(^\s+)|(\s+$)//g;
+		
 		my $correct_modulus  = $self->make_bigint_from_node(
 			$result->{modulus},
 			fallback      => $result->{hexModulus},
@@ -355,7 +299,7 @@ SPARQL
 			);
 		next RESULT
 			unless $correct_exponent == $self->cert_exponent;
-
+		
 		$self->validation(VALIDATION_WEBID);
 		$self->subject_uri($uri);
 		
@@ -459,9 +403,9 @@ sub subject_model
 	my ($self) = shift;
 	if (@_)
 	{
-		$MODEL->{ refaddr($self) } = shift;
+		$MODEL->{ $self->object_id } = shift;
 	}
-	return $MODEL->{ refaddr($self) };
+	return $MODEL->{ $self->object_id };
 }
 
 # Documentation in Advanced.pod
@@ -482,17 +426,17 @@ sub session
 	
 	if (@_)
 	{
-		$SESSION->{ refaddr($self) } = shift;
+		$SESSION->{ $self->object_id } = shift;
 	}
-
-	unless (defined $SESSION->{ refaddr($self) })
+	
+	unless (defined $SESSION->{ $self->object_id })
 	{
 		my $s = CGI::Session->new('driver:file', undef, {Directory => File::Spec->tmpdir});
 		$s->expire('+1h');
-		$SESSION->{ refaddr($self) } = $s;
+		$SESSION->{ $self->object_id } = $s;
 	}
 	
-	return $SESSION->{ refaddr($self) };
+	return $SESSION->{ $self->object_id };
 }
 
 # Documentation in Advanced.pod
@@ -513,7 +457,11 @@ sub get_trine_model
 	$ua->default_headers->push_header('Accept' => "application/rdf+xml, text/turtle, application/x-turtle, application/xhtml+xml;q=0.9, text/html;q=0.9, */*;q=0.1");
 	my $response = $ua->get($uri);
 	return unless $response->is_success && length $response->content;
-	my $model = rdf_parse($response);
+	my $model = rdf_parse(
+		$response->decoded_content,
+		type  => scalar($response->content_type),
+		base  => ($response->base || $uri),
+	);
 	
 	$self->session->param($uri, rdf_string($model, 'ntriples'));
 	$self->session->flush;
@@ -528,7 +476,11 @@ sub make_bigint_from_node
 	
 	if ($node->is_literal)
 	{
-		if ($node->literal_datatype eq 'http://www.w3.org/ns/auth/cert#hex')
+		# HACK to make MyProfile WebIDs parseable (missing the xsd namespace)
+		$node->[2] =~ s@^xsd:@http://www.w3.org/2001/XMLSchema#@;
+		
+		if ( $node->literal_datatype eq 'http://www.w3.org/ns/auth/cert#hex'
+		or   $node->literal_datatype eq 'http://www.w3.org/2001/XMLSchema#hexBinary' )
 		{
 			my $hex = $node->literal_value;
 			$hex =~ s/[^0-9A-F]//ig;
@@ -598,6 +550,120 @@ sub execute_query
 
 __END__
 
+=head1 NAME
+
+CGI::Auth::FOAF_SSL - authentication using WebID (FOAF+SSL)
+
+=head1 SYNOPSIS
+
+  use CGI qw(:all);
+  use CGI::Auth::FOAF_SSL;
+  
+  my $auth = CGI::Auth::FOAF_SSL->new_from_cgi;
+  
+  print header(-type=>'text/html', -cookie=>$auth->cookie);
+  
+  if (defined $auth && $auth->is_secure)
+  {
+    if (defined $auth->subject)
+    {
+      printf("<p>Hello <a href='%s'>%s</a>!</p>\n",
+             escapeHTML($auth->subject->homepage),
+             escapeHTML($auth->subject->name));
+    }
+    else
+    {
+      print "<p>Hello!</p>\n";
+    }
+  }
+  else
+  {
+    print "<p>Greetings stranger!</p>\n";
+  }
+
+=head1 DEPRECATION
+
+CGI::Auth::FOAF_SSL was the original WebID module for Perl, but it is
+now deprecated in favour of L<Web::ID>.
+
+L<Web::ID> has a cleaner interface and is less CGI-specific. It should
+work equally well in other HTTPS contexts. It has L<Plack> middleware
+(but its core does not rely on Plack). Use it.
+
+=head1 DESCRIPTION
+
+FOAF+SSL (a.k.a. WebID) is a simple authentication scheme described
+at L<http://esw.w3.org/topic/foaf+ssl>. This module implements the server
+end of FOAF+SSL in Perl.
+
+It is suitable for handling authentication using FOAF+SSL over HTTPS.
+Your web server needs to be using HTTPS, configured to request client
+certificates, and make the certificate PEM available to your script. If you
+are using Apache, this means that you want to set the following
+directives in your SSL virtual host setup:
+
+ SSLEngine on
+ SSLVerifyClient optional_no_ca
+ SSLVerifyDepth  1
+ SSLOptions +StdEnvVars +ExportCertData
+
+=head2 Configuration
+
+=over 4
+
+=item * C<< $CGI::Auth::FOAF_SSL::ua_string = 'MyTool/1.0' >>
+
+Set the User-Agent string for any HTTP requests.
+
+=back
+
+=head2 Constructors
+
+=over 4
+
+=item * C<< new($pem_encoded) >>
+
+Performs FOAF+SSL authentication on a PEM-encoded key. If authentication is
+completely unsuccessful, returns undef. Otherwise, returns a CGI::Auth::FOAF_SSL
+object. Use C<is_secure> to check if authentication was I<completely> successful.
+
+You probably want to use C<new_from_cgi> instead.
+
+(DER encoded certificates should work too.)
+
+=item * C<< new_from_cgi($cgi_object) >>
+
+Performs FOAF+SSL authentication on a CGI object. This is a wrapper around
+C<new> which extracts the PEM-encoded client certificate from the CGI
+request. It has the same return values as C<new>.
+
+If $cgi_object is omitted, uses C<< CGI->new >> instead.
+
+=back
+
+=head2 Public Methods
+
+=over 4
+
+=item * C<< is_secure >>
+
+Returns true iff the FOAF+SSL authentication process was completely successful.
+
+=item * C<< subject >>
+
+Returns a L<CGI::Auth::FOAF_SSL::Agent> object which represents the subject
+of the certificate. 
+
+This method has aliases C<agent> and C<certified_thing> for back-compat
+reasons.
+
+=item * C<< cookie >>
+
+HTTP cookie related to the authentication process. Sending this to the
+client isn't strictly necessary, but it allows for a session to be
+established, greatly speeding up subsequent accesses. See also the
+COOKIES section of this documentation.
+
 =back
 
 =head1 COOKIES
@@ -683,7 +749,7 @@ Toby Inkster, E<lt>tobyink@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2009-2011 by Toby Inkster
+Copyright (C) 2009-2012 by Toby Inkster
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
