@@ -1,9 +1,10 @@
 package CGI::Auth::FOAF_SSL::Agent;
 
-BEGIN {
-	$CGI::Auth::FOAF_SSL::Agent::AUTHORITY = 'cpan:TOBYINK';
-	$CGI::Auth::FOAF_SSL::Agent::VERSION   = '1.921_00';
-}
+use RDF::Query;
+use RDF::Query::Client;
+use RDF::Trine;
+
+our $VERSION = '1.003';
 
 sub new
 {
@@ -13,7 +14,6 @@ sub new
 	$this->{'identity'}    = shift;
 	$this->{'model'}       = shift;
 	$this->{'endpoint'}    = shift;
-	$this->{'WebID'}       = shift;
 
 	bless $this, $class;
 }
@@ -40,16 +40,42 @@ sub _getter
 {
 	my $this  = shift;
 	my $key   = shift;
-	my @preds = map { RDF::Trine::Node::Resource->new($_) } @_;
+	my @preds = @_;
 	
-	unless ($this->{$key})
+	PREDICATE: foreach my $p (@preds)
 	{
-		$this->{$key} = [ $this->{WebID}->get(@preds) ];
+		last PREDICATE
+			if defined $this->{ $key };
+		
+		my $query_string = sprintf("SELECT ?x WHERE { <%s> <%s> ?x . } ORDER BY ?x", $this->identity, $p);
+		my $results;
+		
+		if (defined $this->model)
+		{
+			my $query = RDF::Query->new($query_string);
+			$results  = $query->execute($this->model);
+		}
+		elsif (defined $this->endpoint)
+		{
+			my $query = RDF::Query::Client->new($query_string);
+			$results  = $query->execute($this->endpoint, {QueryMethod=>'POST'});
+		}
+		
+		RESULT: while (my $row = $results->next)
+		{
+			last RESULT
+				if defined $this->{ $key };
+			
+			my $node = $row->{'x'};
+		
+			if (defined $node and $node->is_resource)
+				{ $this->{ $key } = $node->uri; }
+			elsif (defined $node and $node->is_literal)
+				{ $this->{ $key } = $node->literal_value; }
+		}
 	}
 	
-	wantarray
-		? @{ $this->{$key} } 
-		: $this->{$key}[0]
+	return $this->{ $key };
 }
 
 sub name
@@ -170,7 +196,7 @@ Toby Inkster, E<lt>tobyink@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2009-2012 by Toby Inkster
+Copyright (C) 2009-2011 by Toby Inkster
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
